@@ -8,60 +8,60 @@ Perl::SAX - Generate SAX events for perl source code (incomplete)
 
 =head1 DESCRIPTION
 
-With the completion of a relatively usful perl parser, in the form of
-L<PPI|PPI>, there has been renewed interest in parsing perl source code
-and "Doing Stuff" with it.
+With the completion of L<PPI> and the potential creation of a viable
+refactoring Perl editor, there has been renewed interest in parsing perl
+source code and "Doing Stuff" with it.
 
 It was felt (actually, it was demanded) that there should be some sort of
-event system that could go through a chunk of perl source code and emit events
-that would be handled by a variety of methods.
+event mechanism that could go through a chunk of perl source code and emit
+events that would be handled by a variety of methods.
 
-Rather than invent my own, it was much easier to hijack SAX for this purpose.
+Rather than invent my own, it was much easier to hijack SAX for this
+purpose.
 
-(The ability to write Acme::PerlML had NOTHING to do with it. I swear!)
-
-Perl::SAX is the result of this need. Starting with a single object of any
-type descended from PPI::Node, Perl::Sax will generate a stream of SAX events.
+C<Perl::SAX> is the result of this need. Starting with a single object of any
+type descended from L<PPI::Node>, C<Perl::SAX> will generate a stream of SAX
+events.
 
 For the sake of compatibility with SAX as a whole, and in the spirit of not
 dictating the default behaviour based on any one use of this event stream,
-the stream of events will be such that it can be passed to XML::SAX::Writer
+the stream of events will be such that it can be passed to L<XML::SAX::Writer>
 and a "PerlML" file will be spat out.
 
-This provides the highest level of details, and allows for a variety of
+This provides the highest level of detail, and allows for a variety of
 different potential uses, relating to both the actual and lexical content
 inside of perl source code.
 
-=head2 Perl::SAX is only a SAX Driver
+=head2 Perl::SAX is just a SAX Driver
 
-Please note that Perl::SAX is B<only> a SAX Driver B<only>. It cannot be used
+Please note that C<Perl::SAX> is B<only> a SAX Driver. It cannot be used
 as a SAX Filter or some other form of SAX Handler, and will die fatally if
 you try, as soon as it recieves a C<start_document> event.
 
+To restart Perl::SAX only B<creates> events, it cannot consume them.
+
 =head2 Current State of Completion
 
-Currently, this incomplete version is in CPAN largely as a placeholder. You
-can create an object, you just can't do anything with it.
-
-But I would expect it to be fleshed out a bit more during the next serious
-chunk of time I get to work on PPI.
+This basic first working version is being uploaded to support the creation
+of an L<Acme::Bleach> rip-off using PerlML.
 
 =cut
 
 use strict;
-use UNIVERSAL 'isa';
 use base 'XML::SAX::Base';
-use Class::Autouse 'XML::SAX::Writer';
+use Params::Util '_INSTANCE';
+use PPI::Util    '_Document';
+eval "use prefork 'XML::SAX::Writer';";
 
 use vars qw{$VERSION};
 BEGIN {
-	$VERSION = '0.03';
+	$VERSION = '0.05';
 }
 
 # While in development, use a version-specific namespace.
 # In theory, this ensures documents are only truly valid with the
 # version they were created with.
-use constant XMLNS => "http://ali.as/xml/schema/experimental/perlml/$VERSION/";
+use constant XMLNS => 'http://ali.as/xml/schema/perlml/$VERSION';
 
 
 
@@ -78,26 +78,25 @@ use constant XMLNS => "http://ali.as/xml/schema/experimental/perlml/$VERSION/";
 
 The C<new> constructor creates a new Perl SAX Driver instance.
 
-If passed no arguments, it creates a new default
-L<XML::SAX::Writer|XML::SAX::Writer> object, which by default will write the
-resulting PerlML file to STDOUT.
+If passed no arguments, it creates a new default L<XML::SAX::Writer> object,
+which by default will write the resulting PerlML file to STDOUT.
 
-If passed an C<Output => $Consumer> argument, this value will be passed along
-to the XML::SAX::Writer constructor. Any value that is legal for the Output
-parameter to XML::SAX::Writer is also legal here.
+If passed an C<Output =E<gt> $Consumer> argument, this value will be passed
+along to the L<XML::SAX::Writer> constructor. Any value that is legal for
+the Output parameter to L<XML::SAX::Writer> is also legal here.
 
-If passed a C<Handler => $Handler> argument, C<$Handler> will be used as the
-SAX Handler directly. Any value provided via Output in this case will be
-ignored.
+If passed a C<Handler =E<gt> $Handler> argument, C<$Handler> will be used
+as the SAX Handler directly. Any value provided via Output in this case will
+be ignored.
 
-Returns a new Perl::SAX object, or C<undef> if you pass an illegal Output
-value, and the XML::SAX::Writer cannot be created.
+Returns a new C<Perl::SAX> object, or C<undef> if you pass an illegal Output
+value, and the L<XML::SAX::Writer> cannot be created.
 
 =cut
 
 sub new {
-	my $class  = shift;
-	my %params = @_;
+	my $class = shift;
+	my %param = @_;
 
 	# Create the empty object
 	my $self = bless {
@@ -107,28 +106,31 @@ sub new {
 		}, $class;
 
 	# Have we been passed a custom handler?
-	if ( $params{Handler} ) {
+	if ( $param{Handler} ) {
 		### It appears there is no way to test the validity of a SAX handler
-		$self->{Handler} = $params{Handler};
+		$self->{Handler} = $param{Handler};
 	} else {
 		# Default to an XML::Writer.
 		# Have we been passed in Consumer for it?
-		if ( $params{Output} ) {
-			$self->{Output} = $params{Output};
+		if ( $param{Output} ) {
+			$self->{Output} = $param{Output};
 		} else {
 			my $Output = '';
 			$self->{Output} = \$Output;
 		}
 
 		# Add the handler for the Output
-		$self->{Handler} = XML::SAX::Writer->new( Output => $self->{Output} ) or return undef;
+		require XML::SAX::Writer;
+		$self->{Handler} = XML::SAX::Writer->new(
+			Output => $self->{Output},
+			) or return undef;
 	}
 
 	# Generate NamespaceURI information?
-	if ( $params{NamespaceURI} ) {
-		if ( length $params{NamespaceURI} > 1 ) {
+	if ( $param{NamespaceURI} ) {
+		if ( length $param{NamespaceURI} > 1 ) {
 			# Custom namespace
-			$self->{NamespaceURI} = $params{NamespaceURI};
+			$self->{NamespaceURI} = $param{NamespaceURI};
 		} else {
 			# Default namespace
 			$self->{NamespaceURI} = XMLNS;
@@ -136,8 +138,8 @@ sub new {
 	}
 
 	# Use a prefix?
-	if ( $params{Prefix} ) {
-		$self->{Prefix} = $params{Prefix};
+	if ( $param{Prefix} ) {
+		$self->{Prefix} = $param{Prefix};
 	}
 
 	$self;
@@ -164,7 +166,7 @@ sub start_document {
 
 sub parse {
 	my $self     = shift;
-	my $Document = isa(ref $_[0], 'PPI::Document') ? shift : return undef;
+	my $Document = _INSTANCE(shift, 'PPI::Document') or return undef;
 
 	# Generate the SAX2 events
 	$self->SUPER::start_document( {} );
@@ -176,18 +178,18 @@ sub parse {
 
 sub _parse {
 	my $self    = shift;
-	my $Element = isa(ref $_[0], 'PPI::Element') ? shift : return undef;
+	my $Element = _INSTANCE(shift, 'PPI::Element') or return undef;
 
 	# Split to the various generic handlers
-	  isa($Element, 'PPI::Token')     ? $self->_parse_token( $Element )
-	: isa($Element, 'PPI::Statement') ? $self->_parse_statement( $Element )
-	: isa($Element, 'PPI::Structure') ? $self->_parse_structure( $Element )
+	  $Element->isa('PPI::Token')     ? $self->_parse_token( $Element )
+	: $Element->isa('PPI::Statement') ? $self->_parse_statement( $Element )
+	: $Element->isa('PPI::Structure') ? $self->_parse_structure( $Element )
 	: undef;
 }
 
 sub _parse_document {
 	my $self     = shift;
-	my $Document = isa(ref $_[0], 'PPI::Document') ? shift : return undef;
+	my $Document = _INSTANCE(shift, 'PPI::Document') or return undef;
 
 	# Generate the SAX2 events
 	my $Element = $self->_element( $Document ) or return undef;
@@ -202,7 +204,7 @@ sub _parse_document {
 
 sub _parse_token {
 	my $self  = shift;
-	my $Token = isa(ref $_[0], 'PPI::Token') ? shift : return undef;
+	my $Token = _INSTANCE(shift, 'PPI::Token') or return undef;
 
 	# Support custom handlers
 	my $method = $self->_tag_method( $Token );
@@ -221,7 +223,7 @@ sub _parse_token {
 
 sub _parse_statement {
 	my $self      = shift;
-	my $Statement = isa(ref $_[0], 'PPI::Statement') ? shift : return undef;
+	my $Statement = _INSTANCE(shift, 'PPI::Statement') or return undef;
 
 	# Support custom handlers
 	my $method = $self->_tag_method( $Statement );
@@ -240,14 +242,14 @@ sub _parse_statement {
 
 sub _parse_structure {
 	my $self      = shift;
-	my $Structure = isa(ref $_[0], 'PPI::Structure') ? shift : return undef;
+	my $Structure = _INSTANCE(shift, 'PPI::Structure') or return undef;
 
 	# Support custom handlers
 	my $method = $self->_tag_method( $Structure );
 	return $self->$method( $Structure ) if $self->can($method);
 
 	# Generate the SAX2 events
-	my $Element = $self->_element( $Structure, {} ) or return undef;
+	my $Element = $self->_element( $Structure ) or return undef;
 	$self->start_element( $Element );
 	foreach my $Child ( $Structure->elements ) {
 		$self->_parse( $Child ) or return undef;		
@@ -280,12 +282,6 @@ sub characters {
 			} );
 }
 
-sub _tag_name {
-	my $tag  = lc ref $_[1];
-	$tag =~ s/::/_/g;
-	substr $tag, 4;
-}
-
 sub _tag_method {
 	my $tag = lc ref $_[1];
 	$tag =~ s/::/_/g;
@@ -294,11 +290,9 @@ sub _tag_method {
 
 sub _element {
 	my $self      = shift;
-	my $LocalName = isa(ref $_[0], 'PPI::Element')
-		? $self->_tag_name(shift)
-		: $_[0]
-		or return undef;
-	my $attr = ref $_[0] eq 'HASH' ? shift : {};
+	my ($LocalName, $attr) = _INSTANCE($_[0], 'PPI::Element')
+		? ($_[0]->_xml_name, $_[0]->_xml_attr)
+		: ($_[0], (ref $_[1] eq 'HASH') ? $_[1] : {});
 
 	# Localise some variables for speed
 	my $NamespaceURI = $self->{NamespaceURI};
@@ -350,20 +344,21 @@ are installation-related.
 
 Bugs should be reported via the CPAN bug tracker at
 
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Perl%3A%3ASAX>
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Perl-SAX>
 
-For other issues, contact the author.
+For other issues, or commercial enhancement or support, contact the author...
 
 =head1 AUTHOR
 
-Adam Kennedy (Maintainer), L<http://ali.as/>, cpan@ali.as
+Adam Kennedy, L<http://ali.as/>, cpan@ali.as
 
 =head1 COPYRIGHT
 
 Thank you to Phase N (L<http://phase-n.com/>) for permitting
 the Open Sourcing and release of this distribution.
 
-Copyright (c) 2004 Adam Kennedy. All rights reserved.
+Copyright (c) 2004 - 2005 Adam Kennedy. All rights reserved.
+
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
